@@ -124,12 +124,12 @@ class ShareMixin(object):
         """Returns a list of shares, transformed through view builder."""
         context = req.environ['manila.context']
 
-        common._validate_pagination_query(req)
-
         search_opts = {}
         search_opts.update(req.GET)
 
         # Remove keys that are not related to share attrs
+        search_opts.pop('limit', None)
+        search_opts.pop('offset', None)
         sort_key = search_opts.pop('sort_key', 'created_at')
         sort_dir = search_opts.pop('sort_dir', 'desc')
 
@@ -174,10 +174,14 @@ class ShareMixin(object):
         if show_count:
             total_count = len(shares)
 
+        limited_list = common.limited(shares, req)
+
         if is_detail:
-            shares = self._view_builder.detail_list(req, shares, total_count)
+            shares = self._view_builder.detail_list(req, limited_list,
+                                                    total_count)
         else:
-            shares = self._view_builder.summary_list(req, shares, total_count)
+            shares = self._view_builder.summary_list(req, limited_list,
+                                                     total_count)
         return shares
 
     def _get_share_search_options(self):
@@ -192,9 +196,10 @@ class ShareMixin(object):
             'is_public', 'metadata', 'extra_specs', 'sort_key', 'sort_dir',
             'share_group_id', 'share_group_snapshot_id', 'export_location_id',
             'export_location_path', 'display_name~', 'display_description~',
-            'display_description', 'limit', 'offset'
+            'display_description'
         )
 
+    @wsgi.Controller.authorize
     def update(self, req, id, body):
         """Update a share."""
         context = req.environ['manila.context']
@@ -218,6 +223,9 @@ class ShareMixin(object):
         except exception.NotFound:
             raise exc.HTTPNotFound()
 
+        update_dict = common.validate_public_share_policy(
+            context, update_dict, api='update')
+
         share = self.share_api.update(context, share, update_dict)
         share.update(update_dict)
         return self._view_builder.detail(req, share)
@@ -228,8 +236,10 @@ class ShareMixin(object):
         share = self._create(req, body)
         return share
 
+    @wsgi.Controller.authorize('create')
     def _create(self, req, body,
-                check_create_share_from_snapshot_support=False):
+                check_create_share_from_snapshot_support=False,
+                check_availability_zones_extra_spec=False):
         """Creates a new share."""
         context = req.environ['manila.context']
 
@@ -237,7 +247,7 @@ class ShareMixin(object):
             raise exc.HTTPUnprocessableEntity()
 
         share = body['share']
-        availability_zone_id = None
+        share = common.validate_public_share_policy(context, share)
 
         # NOTE(rushiagr): Manila API allows 'name' instead of 'display_name'.
         if share.get('name'):
@@ -257,6 +267,7 @@ class ShareMixin(object):
                {'share_proto': share_proto, 'size': size})
         LOG.info(msg, context=context)
 
+        availability_zone_id = None
         availability_zone = share.get('availability_zone')
         if availability_zone:
             try:
@@ -296,6 +307,7 @@ class ShareMixin(object):
 
         share_network_id = share.get('share_network_id')
 
+        parent_share_type = {}
         if snapshot:
             # Need to check that share_network_id from snapshot's
             # parents share equals to share_network_id from args.
@@ -303,6 +315,8 @@ class ShareMixin(object):
             # share_network_id of parent share.
             parent_share = self.share_api.get(context, snapshot['share_id'])
             parent_share_net_id = parent_share.instance['share_network_id']
+            parent_share_type = share_types.get_share_type(
+                context, parent_share.instance['share_type_id'])
             if share_network_id:
                 if share_network_id != parent_share_net_id:
                     msg = ("Share network ID should be the same as snapshot's"
@@ -367,6 +381,24 @@ class ShareMixin(object):
                     'driver_handles_share_servers is true.')
             raise exc.HTTPBadRequest(explanation=msg)
 
+        type_chosen = share_type or parent_share_type
+        if type_chosen and check_availability_zones_extra_spec:
+            type_azs = type_chosen.get(
+                'extra_specs', {}).get('availability_zones', '')
+            type_azs = type_azs.split(',') if type_azs else []
+            kwargs['availability_zones'] = type_azs
+            if (availability_zone and type_azs and
+                    availability_zone not in type_azs):
+                msg = _("Share type %(type)s is not supported within the "
+                        "availability zone chosen %(az)s.")
+                type_chosen = (
+                    req_share_type or "%s (from source snapshot)" % (
+                        parent_share_type.get('name') or
+                        parent_share_type.get('id'))
+                )
+                payload = {'type': type_chosen, 'az': availability_zone}
+                raise exc.HTTPBadRequest(explanation=msg % payload)
+
         if share_type:
             kwargs['share_type'] = share_type
         new_share = self.share_api.create(context,
